#include "board.h"
#include "libmy/adrv.h"
#include "libmy/bg.h"
#include "libmy/gbfs.h"
#include "libmy/key.h"
#include "libmy/spr.h"


//---------------------------------------------------------------------------
ST_BOARD Board;


// libmy/adrv.c
extern ST_ADRV Adrv;

//---------------------------------------------------------------------------
EWRAM_CODE void BoardInit()
{
	_Memset(&Board, 0x00, sizeof(ST_BOARD));
	BoardDrawInit();

	u16 cnt = GbfsGetArcCnt();

	if(cnt == 0)
	{
		SystemError("[Err] ViewInit ArcCnt == 0");
	}

	Board.cnt = cnt;
}
//---------------------------------------------------------------------------
EWRAM_CODE void BoardDraw()
{
	BoardDrawSel();
	BoardDrawOctave();
}
//---------------------------------------------------------------------------
EWRAM_CODE void BoardDrawInit()
{
	BgAsciiDrawStr(0, 0, "ONE Music Test.");
	BgAsciiDrawStr(1, 2, "+Channel 1/2.");
	BgAsciiDrawStr(1, 3, "+Interrpt TIMER3 Use.");

	u16 chr = 0;
	u16 x, y;

	for(y=0; y<20; y++)
	{
		for(x=0; x<30; x++)
		{
			Bg1DrawChr(x, y, chr++);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void BoardDrawSel()
{
	GbfsGetSafePointer2(Board.num);
	char* pName = GbfsGetFileName();


	BgAsciiDrawStr(1, 13, "Select File: ");

	                   // "123456789012345678901234567890"
	BgAsciiDrawStr(2, 14, "                              ");
	BgAsciiDrawStr(2, 14, "> ");
	BgAsciiDrawStr(4, 14, pName);
}
//---------------------------------------------------------------------------
EWRAM_CODE void BoardDrawOctave()
{
	//               0  1  2  3  4  5  6  7  8  9 10 11
	//               c c+  d d+  e  f f+  g g+  a a+  b
	const u8 x[] = { 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1};
	const u8 y[] = { 0, 2, 4, 6, 8,12,14,16,18,20,22,24};

	u16 oct, note;
	u16 i;

	for(i=0; i<ACRV_MAX_TRACK_CNT-1; i++)
	{
		ST_ADRV_STATUS* p = &Adrv.status[i];

		oct  = Div(p->note, 12);
		note = DivMod(p->note, 12);

		if(p->gateWork == 0)
		{
			SprSetCur(i, 240, 160);
		}
		else
		{
			SprSetCur(i, 42+ x[note]*16 + i*3, 9 + oct * 28 + y[note]);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void BoardExec()
{
	ST_BOARD* p = &Board;
	u16 key = KeyGetRep();

	if(key & KEY_LEFT)
	{
		if(p->num == 0)
		{
			return;
		}

		p->num--;
	}
	else if(key & KEY_RIGHT)
	{
		if(p->num+1 >= p->cnt)
		{
			return;
		}

		p->num++;
	}
	else if(key & KEY_A)
	{
		p->isSel = TRUE;
	}
	else if(key & KEY_B)
	{
		AdrvStop();
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void BoardExecPlay()
{
	u8* pData = GbfsGetSafePointer2(Board.num);

	AdrvSetData(pData);

	AdrvStop();
	AdrvPlay();

	Board.isSel = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool BoardIsSel()
{
	return Board.isSel;
}
