#include "ac.h"
#include "matuyosi.c"

//---------------------------------------------------------------------------
ST_AC  Ac;
ST_LEX Lex;


//---------------------------------------------------------------------------
const char* DebugCmd[] = {
	"C2",
	"Cs2",
	"D2",
	"Ds2",
	"E2",
	"F2",
	"Fs2",
	"G2",
	"Gs2",
	"A2",
	"As2",
	"B2",
	"C3",
	"Cs3",
	"D3",
	"Ds3",
	"E3",
	"F3",
	"Fs3",
	"G3",
	"Gs3",
	"A3",
	"As3",
	"B3",
	"C4",
	"Cs4",
	"D4",
	"Ds4",
	"E4",
	"F4",
	"Fs4",
	"G4",
	"Gs4",
	"A4",
	"As4",
	"B4",
	"C5",
	"Cs5",
	"D5",
	"Ds5",
	"E5",
	"F5",
	"Fs5",
	"G5",
	"Gs5",
	"A5",
	"As5",
	"B5",
	"C6",
	"Cs6",
	"D6",
	"Ds6",
	"E6",
	"F6",
	"Fs6",
	"G6",
	"Gs6",
	"A6",
	"As6",
	"B6",
	"C7",
	"Cs7",
	"D7",
	"Ds7",
	"E7",
	"F7",
	"Fs7",
	"G7",
	"Gs7",
	"A7",
	"As7",
	"B7",
	"CMD_DUMMY",
	"CMD_R",
	"CMD_Q",
	"CMD_P",
	"CMD_W",
	"CMD_S", 
	"CMD_D",
	"CMD_ECHO",
	"CMD_ECHOVOL",
	"CMD_AND",
	"CMD_TAI",
	"CMD_LOOPBEGIN",
	"CMD_LOOPEND",
	"CMD_VOLUME",
	"CMD_TONE",
	"CMD_TRACKLOOP",
	"CMD_TRACKEND",

	"CMD_MATUYOSI_SW",
	"CMD_MATUYOSI_PL",
	"CMD_MATUYOSI_EV",
	"CMD_MATUYOSI_VL",
	"CMD_MATUYOSI_SP",

	"CMD_DUMMY_END",
};
//---------------------------------------------------------------------------
int main(int argc, char** argv)
{
	printf("AC : ANDrv tiny Compiler (c)2003 AND,MAGiCTOUCH.\n");
	printf("     Modified by akkera102\n");

	s32 i;
	char savName[64];

	for(i=0; i<2; i++)
	{
		u8* pBuf = (u8*)songlist[i];

		AcInit();
		AcExec(pBuf);

		sprintf(savName, "song%02d.mda", i);
		AcSave(savName);

		printf("%s Compile Done.\n", savName);
	}

	return 0;
}
//---------------------------------------------------------------------------
void AcInit()
{
	memset(&Ac, 0x00, sizeof(ST_AC));

	s32 i;

	for(i=0; i<AC_MAX_TRACK_CNT; i++)
	{
		Ac.track[i].length    = 8;
		Ac.track[i].octave    = 4;
		Ac.track[i].volume    = 15;
		Ac.track[i].isUse     = FALSE;
		Ac.track[i].allLength = 0;
		Ac.track[i].codeCnt   = 0;
	}
}
//---------------------------------------------------------------------------
void AcSave(char* srcName)
{
	// Z[ut@CJ܂
	FILE* fp = fopen(srcName, "wb");
	if(fp == NULL)
	{
		printf("Save File Open Error : %s\n", srcName);
		exit(1);
	}



	s32  i, j;

	// Headero
	ST_AC_HEADER acHeader;

	// ID
	strncpy(acHeader.id, "ANDRIVER", 8);

	// OFFSET
	s32 cnt = sizeof(ST_AC_HEADER);

	for(i=0; i<AC_MAX_TRACK_CNT; i++)
	{
		if(Ac.track[i].isUse == TRUE)
		{
			acHeader.offset[i] = cnt;
		}
		else
		{
			acHeader.offset[i] = AC_NO_DATA;
		}

		cnt += Ac.track[i].codeCnt;
	}
	fwrite(&acHeader, sizeof(ST_AC_HEADER), 1, fp);

	// DATA
	for(i=0; i<AC_MAX_TRACK_CNT; i++)
	{
		if(Ac.track[i].isUse == FALSE)
		{
			continue;
		}

//		printf("//---------------------------- \n");
//		printf("Track No     = %d\n", i);
//		printf("Track Length = %d\n", Ac.track[i].allLength);

		for(j=0; j<Ac.track[i].codeCnt; j+=2)
		{
			u8 cmd   = Ac.track[i].codeBuf[j+0];
			u8 param = Ac.track[i].codeBuf[j+1];

			fwrite(&cmd,   sizeof(u8), 1, fp);
			fwrite(&param, sizeof(u8), 1, fp);

			assert(cmd < CMD_DUMMY_END);
//			printf("%15s(%2x), %2x\n", DebugCmd[cmd], cmd , param);
		}
	}

	fclose(fp);
}
//---------------------------------------------------------------------------
u8 AcCalcOctave(u16 note)
{
	//                     a   b  c  d  e  f  g
	const char offset[] = {9, 11, 0, 2, 4, 5, 7};

	ST_AC_TRACK* p = &Ac.track[Ac.curTrack];
	return (u8)(p->octave * AC_MAX_SCALE_CNT + offset[note]);
}
//---------------------------------------------------------------------------
void AcPutCmd(u16 cmd, u16 param)
{
	AcChkCmd(cmd, param);


	ST_AC_TRACK* p = &Ac.track[Ac.curTrack];

	if(p->codeCnt+2 >= AC_MAX_CODE_BUF_CNT)
	{
		printf("Error %d Track codeBuf full\n", Ac.curTrack);
		exit(1);
	}

	p->codeBuf[ p->codeCnt+0 ] = cmd;
	p->codeBuf[ p->codeCnt+1 ] = param;

	p->codeCnt += 2;
}
//---------------------------------------------------------------------------
void AcPutNote(u16 cmd, u16 length)
{
	AcPutCmd(cmd, length);
	AcAddNote(length);
}
//---------------------------------------------------------------------------
void AcAddNote(u16 length)
{
	ST_AC_TRACK* p = &Ac.track[Ac.curTrack];

	p->allLength += length;
}
//---------------------------------------------------------------------------
void AcChkCmd(u16 cmd, u16 param)
{
	switch(cmd)
	{
	case CMD_AND:
		// EMPTY  p[^Ȃ
		break;

	case CMD_LOOPBEGIN:
		// EMPTY  p[^Ȃ
		break;

	case CMD_LOOPEND:
		if(AcChkCmdSub(param, 0, 16) == FALSE) goto Err;
		break;

	case CMD_D:
		if(AcChkCmdSub(param, 0, 512) == FALSE) goto Err;		// TODO l͓K
		break;

	case CMD_VOLUME:
		if(AcChkCmdSub(param, 0, 15) == FALSE) goto Err;
		break;

	case CMD_Q:
		if(AcChkCmdSub(param, 0, 16) == FALSE) goto Err;
		break;

	case CMD_TONE:
		// EMPTY  GBAŎgpĂȂ
		break;

	case CMD_W:
		if(AcChkCmdSub(param, 0, 3) == FALSE) goto Err;
		break;

	case CMD_S:
		if(AcChkCmdSub(param, 0, 7) == FALSE) goto Err;
		break;

	case CMD_P:
		if(AcChkCmdSub(param, 0, 3) == FALSE) goto Err;
		break;

	case CMD_ECHO:
		if(AcChkCmdSub(param, 0, 16) == FALSE) goto Err;
		break;

	case CMD_ECHOVOL:
		if(AcChkCmdSub(param, 0, 15) == FALSE) goto Err;
		break;

	default:
		// TODO ErrChk
		break;
	}

	return;

Err:
	printf("Error Param: %15s(%2x), %2x\n", DebugCmd[cmd], cmd , param);
	exit(1);
}
//---------------------------------------------------------------------------
bool AcChkCmdSub(u16 param, u16 min, u16 max)
{
	return (param >= min && param <= max) ? TRUE : FALSE;
}
//---------------------------------------------------------------------------
void AcSetCurTrack(u32 trackNum)
{
	if(trackNum > 4)
	{
		printf("Error trackNum: %d\n", trackNum);
		exit(1);
	}

	Ac.curTrack = trackNum;
	Ac.track[Ac.curTrack].isUse = TRUE;
}
//---------------------------------------------------------------------------
void AcSetOctave(s32 octave)
{
	ST_AC_TRACK* p = &Ac.track[Ac.curTrack];

	if(octave < 0 || octave > 5)
	{
		printf("Error Octave: %d\n", octave);
		exit(1);
	}

	p->octave = octave;
}
//---------------------------------------------------------------------------
s32 AcGetOctave()
{
	return Ac.track[Ac.curTrack].octave;
}
//---------------------------------------------------------------------------
void AcExec(u8* pBuf)
{
	u8* pCur = pBuf;
	u8  oct;

	for(;;)
	{
		u8 cmd   = *pCur++;
		u8 param = *pCur++;

		switch(cmd)
		{
		case C:
			oct = AcCalcOctave('c' - 'a');
			AcPutNote(oct, param);
			break;

		// case CS:
		case DF:
			oct = AcCalcOctave('c' - 'a') + 1;
			AcPutNote(oct, param);
			break;

		case D:
			oct = AcCalcOctave('d' - 'a');
			AcPutNote(oct, param);
			break;

		// case DS:
		case EF:
			oct = AcCalcOctave('d' - 'a') + 1;
			AcPutNote(oct, param);
			break;

		case E:
			oct = AcCalcOctave('e' - 'a');
			AcPutNote(oct, param);
			break;

		case F:
			oct = AcCalcOctave('f' - 'a');
			AcPutNote(oct, param);
			break;

		// case FS:
		case GF:
			oct = AcCalcOctave('f' - 'a') + 1;
			AcPutNote(oct, param);
			break;

		case G:
			oct = AcCalcOctave('g' - 'a');
			AcPutNote(oct, param);
			break;

		// case GS:
		case AF:
			oct = AcCalcOctave('g' - 'a') + 1;
			AcPutNote(oct, param);
			break;

		case A:
			oct = AcCalcOctave('a' - 'a');
			AcPutNote(oct, param);
			break;

		// case AS:
		case BF:
			oct = AcCalcOctave('a' - 'a') + 1;
			AcPutNote(oct, param);
			break;

		case B:
			oct = AcCalcOctave('b' - 'a');
			AcPutNote(oct, param);
			break;




		case R:
			AcPutCmd(CMD_R, param);
			break;

		case O:
			AcSetOctave(param);
			break;

		case LP:
			AcPutCmd(CMD_TRACKLOOP, 0);
			break;

		case ED:
			AcPutCmd(CMD_TRACKEND, 0);
			break;

		case SW:
			AcPutCmd(CMD_MATUYOSI_SW, param);
			break;

		case PL:
			AcPutCmd(CMD_MATUYOSI_PL, param);
			break;

		case EV:
			AcPutCmd(CMD_MATUYOSI_EV, param);
			break;

		case VL:
			AcPutCmd(CMD_MATUYOSI_VL, param);
			break;

		case SP:
			AcPutCmd(CMD_MATUYOSI_SP, param);
			break;

		case XX:
			AcSetCurTrack(param);
			break;

		case ZZ:
			return;

		// `f[^
		default:
			printf("undef cmd %d %d \n", cmd, param);
			exit(1);

		} // switch
	} // for
}
